@@ -17,5 +17,5 @@
 package com.android.apksig.util;
 
 public interface RunnablesProvider {
-    Runnable getRunnable();
+    Runnable createRunnable();
 }
